#ifndef FINDSTUFF_HH
#define FINDSTUFF_HH
#include <vector>

#include "DatEnv.hh"
#include "TrigClient.hh"
#include "TSeries.hh"
#include "Time.hh"
#include "Interval.hh"
#include "OperStateCondList.hh"



class findChannelStat;
typedef std::vector<findChannelStat>    vfChanStat;
typedef vfChanStat::const_iterator      vfCSc_iter;

typedef std::pair<int, int>             prInt;
typedef std::vector<prInt>              vprInt;

class findStuff:public DatEnv, TrigClient {
private:
  /// Channel info
  vfChanStat             mChans;
  /// Holders for all mChan's TSeries data
  TSeries              **mTS;
  /// Stride parameter
  Interval               mStride;
  /// Check for skips in the data
  Time                   mPrevious, mCurrent;
  std::vector<std::pair<Time,Interval> > mSkips;
  /// Waiting period for data to recover (in seconds)
  int                    mWait;
  /// Size of glitch to trigger off of
  double                 mSize;
  /// Configuration and Output file names
  std::string            mConfig;
  /// Provides conditions which must be satisfied to perform analysis
  OperStateCondList      mOSC;
  /// Read the configuration file
  void                   ReadConfig();
  /// Reset the channel access and parameters
  void                   Reset();
  /// Check that an argument follows an option
  bool                   chkNextArg(int i, const char *argv[], int argc);
  /// Print Usage info
  int                    Usage(const char *progName);
  /// Print Help info
  int                    Help();
//    /// Generates trigger information
//    int genTrigger();

public:
  /// Constructor
  findStuff(int argc, const char *argv[]);
  /// Destructor
  ~findStuff();
  /// Processes the data
  void ProcessData();
  /// Handles interrupt signals from users
  void Attention();
};

/// Contains all data necessary for a channel
class findChannelStat {
  friend class findStuff;
private:
  /// Name of the Channel
  std::string            name;
  /// Channel means, deviations
  double                 mean, mean2, sdev, adev;
  /// m..1 is based on SDev, m..2 is based on Avg Dev.
  long int               max1, min1, max2, min2;
  /// Thresholds on time series values
  long int               refMax, refMin;
  /// Status of current trigger
  int                    wait;
  /// Start time of the trigger
  Time                   startTime;
  /// Test whether the channel is in a triggered state
  int                    checkLimits();
  /// Set the maximum threshold on the time series
  void                   setMax(float Limitmax);
  /// Set the minimum threshold on the time series
  void                   setMin(float Limitmin);

public:
  /// Dummy constructor
  findChannelStat(void) {std::cerr <<"Invalid constructor call"<<std::endl;}
  /// Standard constructor
  explicit findChannelStat(std::string chName, float Limit=5.0): 
    name(chName),mean(0.0),mean2(0.0),sdev(0.0),adev(0.0),max1(0),min1(0),
    max2(0),min2(0) { }
  findChannelStat(std::string chName, double chMean, double chMean2,
		  double chSDev, double chADev, long int chMax, long int chMin,
		  float Limit=5.0): 
    name(chName),mean(chMean),mean2(chMean2),sdev(chSDev),adev(chADev),
    refMax(chMax),refMin(chMin) {setLimits(Limit);}
  /// Copy Constructor
  findChannelStat(const findChannelStat &x): name(x.name),mean(x.mean),
      mean2(x.mean2),sdev(x.sdev),adev(x.adev),max1(x.max1),min1(x.min1),
      max2(x.max2),min2(x.min2),refMax(x.refMax),refMin(x.refMin) {}
  /// Destructor
  ~findChannelStat(void){}
  /// Reset the channel's threshold limits
  void setLimits(float Limitmin, float Limitmax=0.0);
  /// Return the upper threshold for a channel
  long int getMax(void) {return max1;}
  /// Return the lower threshold for a channel
  long int getMin(void) {return min1;}
  /// Return the name of a channel
  std::string &getName(void) {return name;}
};
#endif //ifndef FINDSTUFF_HH


