#ifndef _TIDIO_H_
#define _TIDIO_H_

#include <stdlib.h>
#include <stdio.h>

#include <signal.h>

#include <iostream>
// #include <sstream>

#include <string>
#include <vector>
#include <queue>
#include <list>

#include <unistd.h>
//#include <stream.h>
#include <strings.h>
#include <sys/types.h>
#include <sys/socket.h>
#include <netinet/in.h>
#include <arpa/inet.h>
#include <netdb.h>
#include <inttypes.h>
#include <fcntl.h>
#include <errno.h>

// TID
#include <errors.h>
#include <TidString.h>

using namespace std;

typedef struct tagtfP {
  unsigned int t,f;
  float P;
} tfP;

typedef struct EventLW {

  unsigned int cid; // channel Id number

  int type;

  unsigned int GPSSec, GPSNSec;
  
  vector<tfP> tfp;

  vector<int> status;

} EventLW;


class TidIO { // class for communication with data server

  bool Alive; // set to 0 after 30 seconds of inactivity
  time_t lastTime;

  bool IServer; // set to 1 if server, 0 if client

  static const int BACKLOG = 5;

  // input buffer
  static const unsigned int INPUT_BUFFER = 1000000;
  char *inbuf;
  unsigned int nowPos; // position of next free character in inbuf

  string hostname; // hostname
  int port; // port number
  string ip; // ip address

  int sock; // socket
  list<int> csock; // specific client socket
  int client_sock_length;
  struct sockaddr_in sock_name;

 public:

  TidIO() {};

  TidIO(int p, bool BLOCK = 1); // server

  TidIO(const char* h, int p); // client

  ~TidIO();

  char* get_ip(struct hostent* he);

  int GetPort () {return port;}

  unsigned int GetConnected () {return csock.size();}

  void Write(TidString buffer); // send data to data server, with some control data
  
  void Write(TidString buffer, int soc); // write to a specific socket

  void WriteS(list<int>::iterator soc, const char *buf, unsigned int len); // just write to socket

  void WriteS(int, const char *buf, unsigned int len);

  list<TidString*>* Read(int csock); // read data and split it into events

  void DumpRead(); // dump data to screen

  int Listen(); // listen, and sets socket number csock

  EventLW* DeMarshal(TidString* s, unsigned int *GPS); // demarshal

  void ReadEventList(list<EventLW*>&, unsigned int *GPSsec);

  bool ReadTableofContent(string* source,
			  list<string>* chnnames,
			  list<unsigned int>* cid,
			  list<unsigned int>* fs,
			  double *TRes,
			  double *T0); // read table of content

  void WritePlain(const TidString); // write plain string + carriage return

  bool ReadPlain(TidString **); // reads plain string, stops at \n; allocates memory for string

  void HangUp ( int so ); // close socket

  void HangUp (); // close instruction socket

  void SocketDump(FILE *); // dump socket into an open stream

  void SocketUnDump(FILE *); // dump from open stream to socket

  void WaitForClient (); // do nothing until a client connects

  void Close(); // close client socket 

  bool IsAlive () {return Alive;}

  void Write2All(char *dat, unsigned int len);
};

void sread(void *ptr, char* s, size_t siz);

void sread(void *ptr, TidString* s, size_t siz); // keep internal pointer

#endif
